#ifndef _TICKYPIP_BOARD_H_
#define _TICKYPIP_BOARD_H_

#include "base.h"

#include <qfile.h>

class Cell;
class Level;

class Board
{
	public:
		Board();
		~Board();

		void setupGrid();
		void clear();

		void setWidth(size_t width);
		void setHeight(size_t height);
		size_t getWidth() const;
		size_t getHeight() const;

		void setWrapped(bool wrapped = true);
		bool isWrapped() const;

		void setCell(size_t x, size_t y, Cell *cell);
		Cell *getCell(size_t x, size_t y) const;
		Cell *getNearCell(const Cell *baseCell, Direction dir) const;

		void setUpdateListener(UpdateListener *listener);
		UpdateListener *getUpdateListener() const;

		void setTotalComputers(size_t count);
		size_t getTotalComputers() const;

		void incNumComputers();
		void decNumComputers();

		bool isLevelCompleted() const;

		void updateCell(Cell *cell);

		void loadLevel(const char *filename, size_t num);
		Level *getLevel() const;

		void randomize();

	private:
		UpdateListener *listener;
		Level *level;
		Cell ***grid;
		bool wrapped;

		size_t width, height;

		size_t totalComputers, activeComputers;
};

#endif /* _TICKYPIP_BOARD_H_ */
