#include "ConnectorCell.h"
#include <qpixmap.h>

ConnectorCell::ConnectorCell()
	: Cell(), type(CONNECTOR_NONE)
{
	int i;

	for (i = 0; i < NUM_DIRECTIONS; i++)
		dirConnectors[i] = false;
}

void
ConnectorCell::setDirection(Direction dir)
{
	int i;

	for (i = 0; i < NUM_DIRECTIONS; i++)
		dirConnectors[i] = false;

	switch (getConnectorType())
	{
		case CONNECTOR_END:
			dirConnectors[dir] = true;
			break;

		case CONNECTOR_LINE:
			dirConnectors[dir] = true;
			dirConnectors[DIR_OPPOSITE(dir)] = true;
			break;

		case CONNECTOR_T:
			for (i = 0; i < NUM_DIRECTIONS; i++)
				dirConnectors[i] = true;

			dirConnectors[DIR_OPPOSITE(dir)] = false;

			break;

		case CONNECTOR_L:
			dirConnectors[dir] = true;
			dirConnectors[DIR_ROTATE_RIGHT(dir)] = true;
			break;

		case CONNECTOR_CROSS:
			for (i = 0; i < NUM_DIRECTIONS; i++)
				dirConnectors[i] = true;

			break;

		case CONNECTOR_NONE:
			break;
	}

	Cell::setDirection(dir);
}

void
ConnectorCell::setConnectorType(ConnectorType type)
{
	this->type = type;
}

ConnectorCell::ConnectorType
ConnectorCell::getConnectorType() const
{
	return type;
}

bool
ConnectorCell::hasConnector(Direction dir) const
{
	return dirConnectors[dir];
}

void
ConnectorCell::paint(QPainter &paint)
{
	Direction dir = getDirection();

	QBrush brush;

	Cell::paint(paint);

	if (isActive())
		brush = Qt::red;
	else
		brush = Qt::black;

	switch (getConnectorType())
	{
		case CONNECTOR_END:
			paintLine(paint, dir, false, brush);
			break;

		case CONNECTOR_LINE:
			paintLine(paint, dir, true, brush);
			break;

		case CONNECTOR_T:
			paintLine(paint, dir, false, brush);
			paintLine(paint, DIR_ROTATE_RIGHT(dir), true, brush);
			break;

		case CONNECTOR_L:
			paintLine(paint, dir, false, brush);
			paintLine(paint, DIR_ROTATE_RIGHT(dir), false, brush);
			break;

		case CONNECTOR_CROSS:
			paintLine(paint, NORTH, true, brush);
			paintLine(paint, EAST,  true, brush);
			break;
			
		default:
			break;
	}
}

void
ConnectorCell::paintLine(QPainter &paint, Direction dir, bool d,
						 const QBrush &brush) const
{
	int x, y, w, h;

	if (d)
	{
		switch (dir)
		{
			case NORTH:
			case SOUTH:
				x = LINE_LEN - 1;
				y = 0;
				w = 2;
				h = HEIGHT;
				break;

			case EAST:
			case WEST:
				x = 0;
				y = LINE_LEN - 1;
				w = WIDTH;
				h = 2;
				break;

			default:
				return;
		}
	}
	else
	{
		switch (dir)
		{
			case NORTH:
				x = LINE_LEN - 1;
				y = 0;
				w = 2;
				h = LINE_LEN + 1;
				break;

			case EAST:
				x = LINE_LEN - 1;
				y = LINE_LEN - 1;
				w = LINE_LEN + 1;
				h = 2;
				break;

			case SOUTH:
				x = LINE_LEN - 1;
				y = LINE_LEN - 1;
				w = 2;
				h = LINE_LEN + 1;
				break;
				
			case WEST:
				x = 0;
				y = LINE_LEN - 1;
				w = LINE_LEN + 1;
				h = 2;
				break;

			default:
				return;
		}
	}

	paint.fillRect(x, y, w, h, brush);
}
