#include "base.h"
#include "LevelsDialog.h"

#include <qdir.h>
#include <qhbox.h>
#include <qlayout.h>
#include <qpushbutton.h>

#ifdef LOCAL_COMPILE
# include <sys/types.h>
# include <dirent.h>
#endif

LevelsDialog::LevelsDialog(QWidget *parent, const char *name, bool modal,
						   WFlags f)
	: QDialog(parent, name, modal, f)
{
	QVBoxLayout *vb;

	setCaption(tr("Select A Level Pack"));
	resize(180, 200);
	move((240 - width()) / 2, (320 - height()) / 2);

	vb = new QVBoxLayout(this, 8);
	vb->setAutoAdd(TRUE);

	listview = new QListView(this);
	listview->setAllColumnsShowFocus(true);
	listview->setSorting(0, TRUE);
	listview->addColumn(tr("Name"), 99);
	listview->addColumn(tr("Levels"));
	listview->addColumn("", 0);
	listview->setColumnAlignment(1, Qt::AlignRight);

	connect(listview, SIGNAL(selectionChanged(QListViewItem *)),
			this,     SLOT(selectionChanged(QListViewItem *)));
	connect(listview, SIGNAL(doubleClicked(QListViewItem *)),
			this,     SLOT(doubleClicked(QListViewItem *)));

	loadLevels();

	/*
	 * XXX Set the first item to be selected. Change this to be the current
	 *     level or last-played level.
	 */
	listview->setSelected(listview->firstChild(), true);
}

const QString
LevelsDialog::getSelectedFilename() const
{
	return curFilename;
}

void
LevelsDialog::loadLevels()
{
	/* XXX Need this hack because my qpe crashes on QDir calls. */
#ifdef LOCAL_COMPILE
	DIR *dir = opendir(DATA_PREFIX "levels");
	struct dirent *ent;

	if (dir == NULL)
	{
		qWarning(tr("Cannot find levels directory!"));
		return;
	}

#else
	QDir dir(DATA_PREFIX "levels");

	if (!dir.exists())
	{
		qWarning(tr("Cannot find levels directory!"));
		return;
	}

	dir.setFilter(QDir::Files);
#endif

	QListViewItem *item;

	/* XXX *cringe* */
#ifdef LOCAL_COMPILE
	while ((ent = readdir(dir)) != NULL)
	{
		if (!strcmp(ent->d_name, "..") || !strcmp(ent->d_name, "."))
			continue;

		QString filename = ent->d_name;
#else
	for (size_t i = 0; i < dir.count(); i++)
	{
		QString filename = dir[i];
#endif

		Level *level = new TPLevel();

		if (level->loadLevelFile(DATA_PREFIX "levels/" + filename) == false)
		{
			delete level;
			
			level = new NetWalkLevel(filename);

			if (level->loadLevelFile(DATA_PREFIX "levels/" +
									 filename) == false)
			{
				delete level;
				level = NULL;
			}
		}

		if (level != NULL)
		{
			item = new QListViewItem(listview,
									 level->getTitle(),
									 QString::number(level->getNumLevels()),
									 filename);
		}

		delete level;
	}
}

void
LevelsDialog::selectionChanged(QListViewItem *item)
{
	curFilename = item->text(2);
}

void
LevelsDialog::doubleClicked(QListViewItem *item)
{
	curFilename = item->text(2);

	setResult(Accepted);
}
