#include "QTPBoard.h"

QTPBoard::QTPBoard(QWidget *parent, const char *name, Tickypip *tickypip)
	: QWidget(parent, name), tickypip(tickypip)
{
	board = new Board();
	board->setUpdateListener(this);

	resetSizes();

	clear();
}

QTPBoard::~QTPBoard()
{
	if (board != NULL)
		delete board;
}

void
QTPBoard::loadLevel(const QString &filename, size_t num)
{
	update();

	getBoard()->loadLevel(filename.latin1(), num);

	tickypip->reset();
}

void
QTPBoard::setBoard(Board *board)
{
	if (this->board != NULL)
		delete this->board;

	this->board = board;
}

Board *
QTPBoard::getBoard() const
{
	return board;
}

void
QTPBoard::clear()
{
	repaint();
}

void
QTPBoard::cellUpdated(int, int, Cell *cell)
{
	QPainter paint(this);

	drawCell(paint, cell);
}

void
QTPBoard::resetSizes()
{
	int width  = board->getWidth()  * Cell::WIDTH  + 3;
	int height = board->getHeight() * Cell::HEIGHT + 3;

	setFixedSize(width, height);
}

void
QTPBoard::drawCell(QPainter &paint, Cell *cell)
{
	if (cell == NULL)
		return;

	int tx = cell->getX() * Cell::WIDTH  + 1;
	int ty = cell->getY() * Cell::HEIGHT + 1;

	paint.translate(tx, ty);
	cell->paint(paint);
	paint.translate(-tx, -ty);
}

void
QTPBoard::drawCells(QPainter &paint)
{
	unsigned int x, y;

	for (y = 0; y < board->getHeight(); y++)
	{
		for (x = 0; x < board->getWidth(); x++)
		{
			Cell *cell = board->getCell(x, y);

			if (cell == NULL)
			{
#if 0
				paint.fillRect(x * Cell::WIDTH + 1, y * Cell::HEIGHT + 1,
							   Cell::WIDTH, Cell::HEIGHT,
							   QColor(230, 230, 230));
#endif
			}
			else
				drawCell(paint, cell);
		}
	}
}

void
QTPBoard::drawBoard(QPainter &paint)
{
	resetSizes();

#if 0
	paint.setPen(Qt::black);
	paint.drawRect(0, 0, w + 2, h + 2);

	paint.setPen(QColor(164, 164, 164));
	paint.drawLine(1,     h + 2, w + 2, h + 2);
	paint.drawLine(w + 2, 1,     w + 2, h + 2);
#endif

	drawCells(paint);
}

void
QTPBoard::paintEvent(QPaintEvent *)
{
	QPainter paint(this);

	drawBoard(paint);
}

void
QTPBoard::mousePressEvent(QMouseEvent *evt)
{
	int x = pos2index(evt->pos().x());
	int y = pos2index(evt->pos().y());
	Cell *cell = board->getCell(x, y);

	if (cell == NULL)
		return;

	if (cell->getType() == Cell::BLANK)
		return;

	cell->rotateLeft();

	tickypip->incMovesCount();

	/* Check if the level is completed. */
	if (board->isLevelCompleted())
		tickypip->levelCompleted();
}

