#include "TPLevel.h"
#include "BlankCell.h"
#include "ComputerCell.h"
#include "ConnectorCell.h"
#include "ServerCell.h"

#include <qtextstream.h>

#define MAGIC "$TICKYPIP 0.1"

TPLevel::TPLevel(Board *board)
	: Level(board), topServerCell(NULL)
{
}

bool
TPLevel::isValidFormat()
{
	QString magic;
	
	f.at(0);

	f.readLine(magic, strlen(MAGIC) + 1);

	if (magic != MAGIC)
		return false;

	return true;
}

bool
TPLevel::readHeader()
{
	QString numLevelStr;
	QTextStream t(&f);
	int numLevels;
	bool ok;

	t.readLine();
	t.readLine();

	setTitle(t.readLine());

	numLevelStr = t.readLine();
	numLevels = numLevelStr.toUInt(&ok);

	if (!ok)
		return false;

	setNumLevels(numLevels);

	return true;
}

bool
TPLevel::setupLevel(size_t level)
{
	if (getBoard() == NULL)
		return false;

	QTextStream t(&f);
	size_t width, height;
	bool wrapped;
	size_t l;

	topServerCell = NULL;

	f.at(0);
	t.readLine();
	t.readLine();
	t.readLine();
	t.readLine();
	t.readLine();

	for (l = 0; l < level; l++)
	{
		if (!readLevelHeader(t, width, height, wrapped))
			return false;

		for (size_t i = 0; i <= height; i++)
			t.readLine();
	}

	if (f.atEnd())
		return false;

	getBoard()->clear();

	readLevelHeader(t, width, height, wrapped);

	getBoard()->setWidth(width);
	getBoard()->setHeight(height);
	getBoard()->setWrapped(wrapped);

	getBoard()->setupGrid();

	for (size_t y = 0; y < height; y++)
	{
		char row[width * 2 + 2];
		char cellOptions;
		char cellType;

		t.readRawBytes(row, width * 2 + 1);

		for (size_t j = 0, x = 0; x < width; j += 2, x++)
		{
			cellType    = row[j];
			cellOptions = row[j + 1];

			addCell(cellType, cellOptions, x, y);
		}

		if (f.atEnd() && y < height - 1)
			return false;
	}

	getBoard()->randomize();

	return true;
}

void
TPLevel::addCell(char cellType, char cellOptions, size_t x, size_t y)
{
	switch (cellType)
	{
		case CELL_BLANK:
			getBoard()->setCell(x, y, new BlankCell());
			break;

		case CELL_COMPUTER:
			getBoard()->setCell(x, y, new ComputerCell());
			getBoard()->setTotalComputers(getBoard()->getTotalComputers() + 1);
			break;

		case CELL_CONNECTOR:
		{
			ConnectorCell::ConnectorType type;

			switch (cellOptions)
			{
				case '0': type = ConnectorCell::CONNECTOR_LINE;  break;
				case '1': type = ConnectorCell::CONNECTOR_L;     break;
				case '2': type = ConnectorCell::CONNECTOR_T;     break;
				case '3': type = ConnectorCell::CONNECTOR_CROSS; break;
				case '4': type = ConnectorCell::CONNECTOR_END;   break;
				default:  type = ConnectorCell::CONNECTOR_END;   break;
			}

			if (topServerCell != NULL && topServerCell->getY() == y - 1 &&
				topServerCell->getX() == x)
			{
				ServerCell *cell = new ServerCell(ServerCell::SERVER_BOTTOM);
				getBoard()->setCell(x, y, cell);
				cell->setConnectorType(type);
				topServerCell->setConnectorType(type);
			}
			else
			{
				ConnectorCell *cell = new ConnectorCell();
				getBoard()->setCell(x, y, cell);
				cell->setConnectorType(type);
			}

			break;
		}

		case CELL_SERVER:
		{
			topServerCell = new ServerCell(ServerCell::SERVER_TOP);
			getBoard()->setCell(x, y, topServerCell);
			break;
		}

		default:
			getBoard()->setCell(x, y, NULL);
			break;
	}

	if (getBoard()->getCell(x, y) != NULL)
	{
		Cell *cell = getBoard()->getCell(x, y);
		cell->setX(x);
		cell->setY(y);
		cell->setBoard(getBoard());
	}
}

bool
TPLevel::readLevelHeader(QTextStream &t, size_t &width, size_t &height,
						 bool &wrapped)
{
	bool ok;
	QStringList strList;

	strList = QStringList::split(" ", t.readLine());

	if (strList.count() != 3)
		return false;

	width = strList[0].toUInt(&ok);
	if (!ok) return false;

	height = strList[1].toUInt(&ok);
	if (!ok) return false;

	wrapped = (bool)strList[2].toUInt(&ok);
	if (!ok) return false;

	return true;
}
