#include "Tickypip.h"

#include <qlayout.h>
#include <qmessagebox.h>
#include <qpushbutton.h>

Tickypip::Tickypip(QWidget *parent, const char *name, WFlags fl)
	: QWidget(parent, name, fl)
{
	setCaption(tr("Tickypip"));

	/* Setup the initial border layout. */
	BorderLayout *l = new BorderLayout(this, 0, -1, "borderlayout");

	
	/* Add the north layout. */
	QBoxLayout *north = new QHBoxLayout(NULL, 5, -1, "north-hbox");
	l->add(north, BorderLayout::North);

	/* Add the "Level Packs" button. */
	QPushButton *button = new QPushButton(tr("Level Packs"), this);
	north->addWidget(button, 1, Qt::AlignLeft);

	connect(button, SIGNAL(clicked()),
			this, SLOT(selectLevelPack()));

	/* Add the level drop-down. */
	levelsCombo = new QComboBox(FALSE, this);
	north->addWidget(levelsCombo, 0, Qt::AlignRight);

	connect(levelsCombo, SIGNAL(activated(int)),
			this, SLOT(levelChanged(int)));


	/* Add the center layout. */
	QBoxLayout *center = new QVBoxLayout(NULL, 0, -1, "center-vbox");
	l->add(center, BorderLayout::Center);

	/* Add the board. */
	board = new QTPBoard(this, NULL, this);
	center->addWidget(board, Qt::AlignCenter);


	/* Add the south layout. */
	QBoxLayout *southWrapper = new QVBoxLayout(NULL, 0, 0, NULL);
	l->add(southWrapper, BorderLayout::South);

	QGridLayout *south = new QGridLayout(southWrapper, 2, 1, -1,
										 "south-grid");
	/* Add the "Next Level" button */
	nextButton = new QPushButton(tr("Next Level"), this, "next-level-button");
	south->addWidget(nextButton, 0, 0, Qt::AlignCenter);
	south->addRowSpacing(0, nextButton->sizeHint().height());

	connect(nextButton, SIGNAL(clicked()),
			this, SLOT(nextLevel()));

	/* Add the moves box */
	QBoxLayout *movesBox = new QHBoxLayout(NULL, 5, -1, "moves-hbox");

	movesBox->setResizeMode(QLayout::Fixed);
	south->addLayout(movesBox, 1, 0);
	movesBox->insertSpacing(0, 20);

	/* Add the "Moves:" label. */
	QLabel *label = new QLabel(tr("Moves:"), this);
	movesBox->addWidget(label, 0, Qt::AlignRight);

	/* Add the moves counter label. */
	movesLabel = new QLabel("0", this);
	movesBox->addWidget(movesLabel);

	l->activate();

	/* Pop up the levels dialog */
	if (!selectLevelPack())
		exit(0);
}

Tickypip::~Tickypip()
{
	delete levelsCombo;
	delete movesLabel;
	delete board;
}

void
Tickypip::reset()
{
	Level *level;

	movesLabel->setText("0");
	levelsCombo->clear();

	level = board->getBoard()->getLevel();

	if (level != NULL)
	{
		for (size_t i = 0; i < level->getNumLevels(); i++)
		{
			levelsCombo->insertItem(tr("Level %1").arg(QString::number(i + 1)));
		}
	}
}

void
Tickypip::levelCompleted()
{
	nextButton->show();

#if 0
	QString message = "Level completed in ";
	message = message + movesLabel->text() + " moves!";

	int curLevel = levelsCombo->currentItem();

	if (curLevel + 1 == (int)board->getBoard()->getLevel()->getNumLevels())
	{
		QMessageBox::information(this, "Tickypip", message, "New Level Pack");

		selectLevelPack();
	}
	else
	{
		QMessageBox::information(this, "Tickypip", message, "Next Level");

		levelChanged(levelsCombo->currentItem() + 1);
	}
#endif
}

void
Tickypip::incMovesCount()
{
	movesLabel->setText(QString::number(movesLabel->text().toInt() + 1));
}

void
Tickypip::levelChanged(int level)
{
	nextButton->hide();
	levelsCombo->setCurrentItem(level);
	movesLabel->setText("0");
	board->getBoard()->getLevel()->setupLevel(level);
	board->repaint();
}

void
Tickypip::nextLevel()
{
	levelChanged(levelsCombo->currentItem() + 1);
}

bool
Tickypip::selectLevelPack()
{
	LevelsDialog levelsDialog(this);
	int r = levelsDialog.exec();

	nextButton->hide();

	if (r)
	{
		QString filename = levelsDialog.getSelectedFilename();

		if (filename == "")
			return false;

		board->loadLevel(DATA_PREFIX "levels/" + filename, 0);

		return true;
	}

	return false;
}

